﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.VisualBasic.FileIO;
using System.Globalization;

namespace GCSOLAR
{
    public partial class FormAbsorptions : Form
    {
        private bool usePureWaterAbs = false;

        public FormAbsorptions()
        {
            InitializeComponent();
        }

        public void setVisibleWaterColumn(bool flag)
        {
            usePureWaterAbs = flag;
        }

        private void FormAbsorptions_Load(object sender, EventArgs e)
        {
            //this.dataGridView1.CellValidating += new DataGridViewCellValidatingEventHandler(dataGridView1_CellValidating);
            //this.dataGridView1.CellEndEdit += new DataGridViewCellEventHandler(dataGridView1_CellEndEdit);

            DataTable table;
            Common.Listing(out table); 
 
            // If user selected pure water as water body, use pure water absorption coefficients and
            // disable buttons that import water absorption coefficients.
            if (usePureWaterAbs)
            {
                double[] abwat = Common.getPureWaterAbs();
                int minwav = Common.minwav;
                int maxwav = Common.maxwav;
                for (int i = minwav; i <= maxwav; i++)
                {
                    int j = i - minwav;
                    // Multiply water Absorption coefficients by 100 to convert units from cm^-1 to m^-1.
                    double absorpWater = abwat[i - 1] * 100.0;
                    table.Rows[j]["Water Absorption Coefficients (m**-1)"] = absorpWater.ToString("F6", CultureInfo.InvariantCulture);
                }
                btnImportAll.Enabled = false;
                btnImportCSV.Enabled = false;
            }

            dataGridView1.DataSource = table;
            dataGridView1.Columns[0].ReadOnly = true;  // Make the wavelength column read-only.
            dataGridView1.Columns[1].ReadOnly = usePureWaterAbs;  // Make the water absorption coefficients column read-only.
        }

        /// <summary>
        /// This methods closes the current form.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// This method updates the absorption coefficients.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            DataTable table = (DataTable) dataGridView1.DataSource;
            int jj = Common.minwav;
            foreach (DataRow row in table.Rows)
            {
                double abs = Convert.ToDouble(row["Water Absorption Coefficients (m**-1)"].ToString());
                // Convert units of abs to cm^-1 by dividing by 100.
                abs = abs / 100.0;
                double eps = Convert.ToDouble(row[Common.contaminantType + " " + Common.contaminantUnits].ToString());
                Common.setEppest(eps, jj - 1);
                Common.setAbwat(abs, jj - 1);
                jj++;
            }
            MessageBox.Show("Data were successfully updated.");
            this.Close();
        }

        /// <summary>
        /// This method validates the values entered in the dataGridView1's cells.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView1_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            // Confirm that the cell is not empty.
            if (string.IsNullOrEmpty(e.FormattedValue.ToString()))
            {
                e.Cancel = true;
                dataGridView1.Rows[e.RowIndex].ErrorText = "Cell must not be empty.";              
                return;
            }

            double value;
            // Confirm that the cell value is double.
            if (!Double.TryParse(e.FormattedValue.ToString(), out value))
            {
                dataGridView1.Rows[e.RowIndex].ErrorText = "Cell value must be a real number.";
                e.Cancel = true;
                return;
            }

            // Check whether the value is non-positive and if so issue an error message.
            if (value <= 0.0)
            {
                dataGridView1.Rows[e.RowIndex].ErrorText = "Cell value must be > 0.0";
                e.Cancel = true;
                return;
            }
        }

        /// <summary>
        /// This method clears any error message from the dataGridView1's row where one of the 
        /// cells has been validated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView1_CellValidated(object sender, DataGridViewCellEventArgs e)
        {
            // Clear the row error in case the user presses ESC.   
            dataGridView1.Rows[e.RowIndex].ErrorText = String.Empty;
        }

        /// <summary>
        /// This is the main validation method for the file with the data to be uploaded/imported
        /// into the program.
        /// </summary>
        /// <param name="table"></param>
        /// <param name="data"></param>
        /// <param name="errorMessage"></param>
        public void validateInputFile(DataTable table, string data, out string errorMessage)
        {
            errorMessage = "";
            int numCols = table.Columns.Count;
            if (data.Equals("All"))
            {
                if (numCols != 3)
                {
                    errorMessage = "Number of columns must be 3.";
                    return;
                }
                checkHeaderWL(table.Columns[0].ToString(), out errorMessage);
                if (errorMessage != "") return;
                checkHeaderWater(table.Columns[1].ToString(), out errorMessage);
                if (errorMessage != "") return;
                checkHeaderContaminant(table.Columns[2].ToString(), out errorMessage);
                if (errorMessage != "") return;
            }
            else
            {
                if (numCols != 2)
                {
                    errorMessage = "Number of columns must be 2.";
                    return;
                }
                if (data.Equals("Water"))
                {
                    checkHeaderWL(table.Columns[0].ToString(), out errorMessage);
                    if (errorMessage != "") return;
                    checkHeaderWater(table.Columns[1].ToString(), out errorMessage);
                    if (errorMessage != "") return;
                }
                if (data.Equals("Contaminant"))
                {
                    checkHeaderWL(table.Columns[0].ToString(), out errorMessage);
                    if (errorMessage != "") return;
                    checkHeaderContaminant(table.Columns[1].ToString(), out errorMessage);
                    if (errorMessage != "") return;
                }
            }

            // Get minimum and maximum wavelengths.
            double minwavel = Common.getMinWave();
            double maxwavel = Common.getMaxWave();

            // Convert wavelengths to strings.
            string minWL = minwavel.ToString("###.0");
            string maxWL = maxwavel.ToString("###.0");

            // Build SQL statement for Select method of DataBase table.
            string selection = "[Wavelength (nm)] >= " + minWL + " AND [Wavelength (nm)] <= " + maxWL;
            DataRow[] result = table.Select(selection);

            // Check that the extracted data from table are valid.
            checkFields(result, data, out errorMessage);
            if (errorMessage != "") return;

            // Check that imported data contains sufficient values for the specified wavelength
            // range.
            checkDataRange(result, out errorMessage);
            if (errorMessage != "") return;
        }

        
        /// <summary>
        /// This method validates the header of the wavelengths column in the input file.
        /// </summary>
        /// <param name="header"></param>
        /// <param name="errorMessage"></param>
        private void checkHeaderWL(string header, out string errorMessage)
        {
            errorMessage = "";
            
            if (string.IsNullOrWhiteSpace(header))
            {
                errorMessage = "Wavelength column header must not be empty.";
            }
            else if (!header.ToLower().Equals("wavelength (nm)"))
            {
                errorMessage = "First column header must be: Wavelength (nm).";
            }
        }

        /// <summary>
        /// This method validates the header of the water absorptions column in the input file.
        /// </summary>
        /// <param name="header"></param>
        /// <param name="errorMessage"></param>
        private void checkHeaderWater(string header, out string errorMessage)
        {
            errorMessage = "";
            
            if (string.IsNullOrWhiteSpace(header))
            {
                errorMessage = "Water column header must not be empty.";
            }
            else if (!header.ToLower().Equals("water absorption coefficients (m**-1)"))
            {
                errorMessage = "Water column header must be: Water Absorption Coefficients (m**-1).";
            }
        }

        /// <summary>
        /// This method validates the header of the contaminant absorptions column in the input file.
        /// </summary>
        /// <param name="header"></param>
        /// <param name="errorMessage"></param>
        private void checkHeaderContaminant(string header, out string errorMessage)
        {
            errorMessage = "";

            if (string.IsNullOrWhiteSpace(header))
            {
                errorMessage = "Contaminant column header must not be empty.";
            }
            else if (!header.ToLower().Equals(Common.contaminantType.ToLower() + " " + Common.contaminantUnits.ToLower()))
            {
                errorMessage = "Contaminant column header must be: " + Common.contaminantType + " " + Common.contaminantUnits;
            }
        }

        /// <summary>
        /// This method validates wavelengths, water absorptions, and contaminat absorptions.
        /// It calls a specific method for each parameter.
        /// </summary>
        /// <param name="results"></param>
        /// <param name="data"></param>
        /// <param name="errorMessage"></param>
        private void checkFields(DataRow[] results, string data, out string errorMessage)
        {
           errorMessage = "";
           double wave1 = 0;
           double wave2 = 0;
           foreach (DataRow row in results)
           {
               string col0 = row["Wavelength (nm)"].ToString();             
               checkField0(col0, out errorMessage);
               if (errorMessage != "") return;
               Double.TryParse(row["Wavelength (nm)"].ToString(), out wave2);               
               if (wave2 <= wave1)
               {                 
                   errorMessage = "Wavelengths must be in ascending order and must not be repeated.";
                   return;
               }
               wave1 = wave2;

               if (data == "All" || data == "Water")
               {
                   string col1 = row["Water Absorption Coefficients (m**-1)"].ToString();
                   checkField1(col1, out errorMessage);
                   if (errorMessage != "") return;
               }
               
               if (data == "All" || data == "Contaminant")
               {
                   string col2 = row[Common.contaminantType +  " " + Common.contaminantUnits].ToString();
                   checkField2(col2, out errorMessage);
                   if (errorMessage != "") return;
               }
               
           }
                        
        }

        /// <summary>
        /// This method verifies that the input wavelengths include all the allowed values 
        /// within the specified range.
        /// </summary>
        /// <param name="results"></param>
        /// <param name="errorMessage"></param>
        private void checkDataRange(DataRow[] results, out string errorMessage)
        {
            errorMessage = "";
            double[] wave = Common.getWave();
            double waveL;
            int iminw = Common.minwav;
            int imaxw = Common.maxwav;
            bool valueFound = false;

            for (int i = iminw; i <= imaxw; i++)
            {
                foreach(DataRow row in results)
                {
                    Double.TryParse(row["Wavelength (nm)"].ToString(), out waveL);
                    if (Math.Abs(waveL - wave[i - 1]) < 0.001)
                    {
                        valueFound = true;
                        break;
                    }
                }
                if (!valueFound)
                {
                    errorMessage = "Some of the allowed wavelengths within the specified range are missing.";
                }
                else
                {
                    valueFound = false;
                }
            }
        }

        /// <summary>
        /// This method validates individual wavelengths.
        /// </summary>
        /// <param name="wl"></param>
        /// <param name="errorMessage"></param>
        private void checkField0(string wl, out string errorMessage)
        {
            errorMessage = "";
            double wavelength;

            if (string.IsNullOrEmpty(wl))
            {
                errorMessage = "Wavelength must not be empty.";
                return;
            }
            else if (wl.Contains(" "))
            {
                errorMessage = "Do not enter spaces for wavelenth.";
                return;
            }
            else if (!Double.TryParse(wl, out wavelength))
            {
                errorMessage = "Wavelength must be a floating value.";
                return;
            }
            else if (wavelength < 280.0 || wavelength > 800.0)
            {
                errorMessage = "Wavelength must be a number between 280 and 800 nm.";
                return;
            }

            bool valueFound = false;
            double[] wave = Common.getWave();
            for (int i = 0; i < wave.Length; i++)
            {
                if (Math.Abs(wavelength - wave[i]) < 0.001)
                {
                    valueFound = true;
                    break;
                }
            }
            if (!valueFound)
            {
                errorMessage = "Wavelength must be one of the allowed values shown under Help.";
                return;
            }


        }

        /// <summary>
        /// This method validates individual water absorptions.
        /// </summary>
        /// <param name="wabs"></param>
        /// <param name="errorMessage"></param>
        private void checkField1(string wabs, out string errorMessage)
        {
            errorMessage = "";
            double waterAbs;

            if (string.IsNullOrEmpty(wabs))
            {
                errorMessage = "Water absorption must not be empty.";
                return;
            }
            else if (wabs.Contains(" "))
            {
                errorMessage = "Do not enter spaces for water absorption.";
                return;
            }
            else if (!Double.TryParse(wabs, out waterAbs))
            {
                errorMessage = "Water absorption must be a real number.";
                return;
            }
            else if (waterAbs <= 0.0)
            {
                errorMessage = "Water absorption must be greater than zero.";
                return;
            }
        }

        /// <summary>
        /// This method validates individual contaminant absorptions.
        /// </summary>
        /// <param name="mabs"></param>
        /// <param name="errorMessage"></param>
        private void checkField2(string mabs, out string errorMessage)
        {
            errorMessage = "";
            double molarAbs;

            if (string.IsNullOrEmpty(mabs))
            {
                errorMessage = "Contaminant absorption must not be empty.";
                return;
            }
            else if (mabs.Contains(" "))
            {
                errorMessage = "Do not enter spaces for contaminant absorption.";
                return;
            }
            else if (!Double.TryParse(mabs, out molarAbs))
            {
                errorMessage = "Contaminant absorption must be a real number.";
                return;
            }
            else if (molarAbs < 0.0)
            {
                errorMessage = "Contaminant absorption must be greater than or equal to zero.";
                return;
            }

        }

       /// <summary>
       /// This method imports data from a CSV file.
       /// </summary>
       /// <param name="data"></param>
        private void importDataCSV(string data)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.InitialDirectory = @"C:\";
            openFileDialog1.Title = "Browse CSV Files";

            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;

            openFileDialog1.DefaultExt = "csv";
            openFileDialog1.Filter = "Text files (*.csv)|*.csv|All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 2;
            openFileDialog1.RestoreDirectory = true;

            openFileDialog1.ReadOnlyChecked = true;
            openFileDialog1.ShowReadOnly = true;

            string path;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                path = openFileDialog1.FileName;
                DataTable table = new DataTable();
               
                using (TextFieldParser csvParser = new TextFieldParser(path))
                {
                    csvParser.CommentTokens = new string[] { "#" };
                    csvParser.SetDelimiters(new string[] { "," });
                    csvParser.HasFieldsEnclosedInQuotes = true;

                    try
                    {
                        // Read and store the column names.
                        string[] headers = csvParser.ReadFields();

                        for (int j = 0; j < headers.Length; j++)
                        {
                            table.Columns.Add(headers[j]);
                        }

                        DataRow dr1 = null;
                        while (!csvParser.EndOfData)
                        {
                            // Read current line fields, pointer moves to the next line.
                            string[] fields = csvParser.ReadFields();
                            dr1 = table.NewRow();
                            for (int j = 0; j < fields.Length; j++)
                            {
                                dr1[headers[j]] = fields[j];
                            }
                            table.Rows.Add(dr1);
                        }
                    }
                    catch (Exception ex)
                    {
                        string message = ex.ToString() + " " + "Reload the CSV file after fixing the error.";
                        MessageBox.Show(message);
                    }
                }

                // Validate input file.
                string errorMessage = "";
                validateInputFile(table, data, out errorMessage);
                if (errorMessage != "")
                {
                    MessageBox.Show("Error: " + errorMessage);
                    return;
                }

                // Get minimum and maximum wavelengths.
                double minwavel = Common.getMinWave();
                double maxwavel = Common.getMaxWave();

                // Convert wavelengths to strings.
                string minWL = minwavel.ToString("###.0");
                string maxWL = maxwavel.ToString("###.0");

                // Build SQL statement for Select method of DataBase table.
                string selection = "[Wavelength (nm)] >= " + minWL + " AND [Wavelength (nm)] <= " + maxWL;
                DataRow[] result = table.Select(selection);
                int irow = 0;
                foreach (DataRow row in result)
                {
                    foreach (DataColumn column in table.Columns)
                    {
                        string colname = column.ColumnName;
                        dataGridView1[colname, irow].Value = row[colname].ToString(); 
                    }
                    irow++;
                }
                MessageBox.Show("Data were successfully imported into the application.");
            }
        }

        /// <summary>
        /// This method handles the import of water absorptions.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnImportCSV_Click(object sender, EventArgs e)
        {
            string data = "Water";
            importDataCSV(data);

        }

        /// <summary>
        /// This method handles the import of contaminant absorptions.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnContaminant_Click(object sender, EventArgs e)
        {
            string data = "Contaminant";
            importDataCSV(data);
        }

        /// <summary>
        /// This method handles the import of all the data.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnImportAll_Click(object sender, EventArgs e)
        {
            string data = "All";
            importDataCSV(data);
        }

        /// <summary>
        /// This methods will handle the help section of the application.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnHelp_Click(object sender, EventArgs e)
        {
            FormHelp fhelp = new FormHelp();
            fhelp.ShowDialog();
        }

        private void FormAbsorptions_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = false;
        }

        


    }
}
