﻿using System;
using System.Data;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using GCSOLAR;

namespace UnitTestGCSolar
{
    [TestClass]
    public class UnitTest1
    {
        /// <summary>
        /// It verifies that no error messages are issued if the input data are valid.
        /// </summary>
        [TestMethod]
        public void TestDataValidationNoErrors()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = {297.5, 300.0, 302.5};
            double[] water = {1.12, 0.9, 0.75};
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

           FormAbsorptions form1 = new FormAbsorptions();
           string data = "All";
           string errorMessage = "";
           form1.validateInputFile(dt, data, out errorMessage);

           Assert.AreEqual("", errorMessage);
        }

        /// <summary>
        /// It verifies that an error message is issued if invalid wavelengths are found
        /// in the data.
        /// </summary>
        [TestMethod]
        public void TestValidWavelengths()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 301.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Wavelength must be one of the allowed values shown under Help.", errorMessage);
        }

        /// <summary>
        /// It will verify that no error message is issued if the wavelengths are not in 
        /// ascending order.
        /// </summary>
        [TestMethod]
        public void TestOrderOfWavelengths()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 302.5, 300.0 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("", errorMessage);
        }

        /// <summary>
        /// It verifies that the correct error message is issued if the water header 
        /// has the wrong format/spelling.
        /// </summary>
        [TestMethod]
        public void TestSpellingWaterHeader()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.5, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (cm**-1)");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (cm**-1)"] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Water column header must be: Water Absorption Coefficients (m**-1).", errorMessage);
        }

        /// <summary>
        /// It verifies that the correct error message is issued if the header for water
        /// absorptions is missing/empty.
        /// </summary>
        [TestMethod]
        public void TestMissingWaterHeader()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.5, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add(" ");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1[" "] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Water column header must not be empty.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that the correct error message is issued if the wavelength header 
        /// has the wrong format/spelling.
        /// </summary>
        [TestMethod]
        public void TestSpellingWavelengthHeader()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.5, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelenngth (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelenngth (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("First column header must be: Wavelength (nm).", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that the correct error message is issued if the water header 
        /// is missing/empty.
        /// </summary>
        [TestMethod]
        public void TestMissingWavelengthHeader()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.5, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add(" ");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1[" "] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[GCSOLAR.Common.contaminantType.Trim() + " Absorption Coefficients (L/(mole cm))"] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Wavelength column header must not be empty.", errorMessage);
        }

        /// <summary>
        /// It verifies that the correct error message is issued if the contaminant header 
        /// is missing/empty.
        /// </summary>
        [TestMethod]
        public void TestMissingContaminantHeader()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.5, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(" ");

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[" "] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Contaminant column header must not be empty.", errorMessage);
        }

        /// <summary>
        /// It verifies that the correct error message is issued if the contaminant header 
        /// has the wrong format/spelling.
        /// </summary>
        [TestMethod]
        public void TestSpellingContaminantHeader()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.5, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };
            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " Extinction Coefficients " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[Common.contaminantType + " Extinction Coefficients " + Common.contaminantUnits] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Contaminant column header must be: " + Common.contaminantType + " " + Common.contaminantUnits, errorMessage);
        }

        /// <summary>
        /// It verifies that the "Some of the allowed wavelengths within the specified range 
        /// are missing" message is issued whenever a wavelength value is missing from the
        /// user-specified wavelength range.
        /// </summary>
        [TestMethod]
        public void TestMissingWavelengthValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                if (i != 2)
                {
                    dr1["Wavelength (nm)"] = wave[i];
                }
                else
                {
                   dr1["Wavelength (nm)"] = DBNull.Value;
                }
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Some of the allowed wavelengths within the specified range are missing.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that the "Water absorption must not be empty" error message gets issued
        /// whenever water absorptions are missing from the data table.
        /// </summary>
        [TestMethod]
        public void TestMissingWaterValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();               
                dr1["Wavelength (nm)"] = wave[i];               
                if (i != 2)
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                }
                else
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = DBNull.Value;
                }
                
                dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Water absorption must not be empty.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that an error message gets issued if contaminant absorptions are 
        /// missing.
        /// </summary>
        [TestMethod]
        public void TestMissingContaminantValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                if (i != 2)
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                }
                else
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = DBNull.Value;
                }

                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Contaminant absorption must not be empty.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that the "Do not enter spaces for water absorption" error message 
        /// gets issued whenever water absorptions are missing from the data table.
        /// </summary>
        [TestMethod]
        public void TestSpacesWaterValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                if (i != 2)
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                }
                else
                {
                    dr1["Water Absorption Coefficients (m**-1)"]  = " ";
                }

                dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Do not enter spaces for water absorption.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that an error message gets issued if contaminant absorptions have 
        /// spaces.
        /// </summary>
        [TestMethod]
        public void TestSpacesContaminantValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                if (i != 2)
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                }
                else
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = " ";
                }

                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Do not enter spaces for contaminant absorption.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that an error message gets issued if water absorptions are
        /// non numeric.
        /// </summary>
        [TestMethod]
        public void TestNonNumericWaterValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                if (i != 2)
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                }
                else
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = "A";
                }

                dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Water absorption must be a real number.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that an error message gets issued if contaminant absorptions are 
        /// non numeric.
        /// </summary>
        [TestMethod]
        public void TestNonNumericContaminantValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                if (i != 2)
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                }
                else
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = "A";
                }
               
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Contaminant absorption must be a real number.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that an error message gets issued if a negative water absorption 
        /// is found in the data.
        /// </summary>
        [TestMethod]
        public void TestNegativeWaterValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];
                if (i != 2)
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                }
                else
                {
                    dr1["Water Absorption Coefficients (m**-1)"] = -1.0;
                }

                dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Water absorption must be greater than zero.", errorMessage.Trim());
        }

        /// <summary>
        /// It verifies that an error message gets issued if a negative contaminant absorption 
        /// is found in the data.
        /// </summary>
        [TestMethod]
        public void TestNegativeContaminantValue()
        {
            Common.maxwav = 10;
            Common.minwav = 8;
            double[] wave = { 297.5, 300.0, 302.5 };
            double[] water = { 1.12, 0.9, 0.75 };
            double[] chem = { 0.4, 0.3, 0.22 };

            DataTable dt = new DataTable();
            DataRow dr1 = null;
            dt.Columns.Add("Wavelength (nm)");
            dt.Columns.Add("Water Absorption Coefficients (m**-1)");
            dt.Columns.Add(Common.contaminantType + " " + Common.contaminantUnits);

            for (int i = 0; i < 3; i++)
            {
                dr1 = dt.NewRow();
                dr1["Wavelength (nm)"] = wave[i];               
                dr1["Water Absorption Coefficients (m**-1)"] = water[i];
                if (i != 2)
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = chem[i];
                }
                else
                {
                    dr1[Common.contaminantType + " " + Common.contaminantUnits] = -1;
                }
                
                dt.Rows.Add(dr1);
            }

            FormAbsorptions form1 = new FormAbsorptions();
            string data = "All";
            string errorMessage = "";
            form1.validateInputFile(dt, data, out errorMessage);

            Assert.AreEqual("Contaminant absorption must be greater than or equal to zero.", errorMessage.Trim());
        }

        /// <summary>
        /// Compute direct photolysis rates of methoxychlor 
        /// (near surface) at latitude 40 degrees north during 
        /// mid-spring as a function of time-of-day.
        /// </summary>
        [TestMethod]
        public void TestRatesLat40Spring()
        {
            double delta = 1.0E-12;
            Common.xlon = 90.0;
            Common.ilatsw = 1;
            Common.ilattm[0] = 40.0;
            Common.ityp = 0;
            Common.aveozo = -1.0;
            Common.ioz = 0;
            Common.q = 0.32;
            Common.deltaz = 0.001;
            Common.elevation = 0.0;
            Common.dinc = 10.0;
            Common.dfinal = 5.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Spring";
            Common.minwav = 1;
            Common.maxwav = 19;

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            int startPos;
            int length;

            string strr1 = dtday.Rows[19]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[20]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[21]["Summary Items"].ToString();           
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);
            
            double r1, r2, r3;
            Double.TryParse(sub1, out r1);    
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(1.27E-07, r1, delta);
            Assert.AreEqual(6.06E-03, r2, delta);
            Assert.AreEqual(1.14E+02, r3, delta);
        }

        /// <summary>
        /// Compute effects of change in stratospheric ozone layer 
        /// change of direct photolysis rate of methoxychlor.
        /// </summary>
        [TestMethod]
        public void TestRatesOzoneLayer()
        {
            double delta = 1.0E-12;
            Common.xlon = 90.0;
            Common.ilatsw = 1;
            Common.ilattm[0] = 40;
            Common.aveozo = 0.20;
            Common.ioz = 1;
            Common.q = 0.32;
            Common.deltaz = 0.001;
            Common.elevation = 0.0;
            Common.dinc = 10.0;
            Common.dfinal = 5.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Spring";
            Common.minwav = 1;
            Common.maxwav = 19;

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            int startPos;
            int length;

            string strr1 = dtday.Rows[19]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[20]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[21]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);

            double r1, r2, r3;
            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(2.59E-07, r1, delta);
            Assert.AreEqual(1.24E-02, r2, delta);
            Assert.AreEqual(5.60E+01, r3, delta);
        }

        /// <summary>
        /// Compute rate constants for methoxychlor at an 
        /// elevation of 4.0 KM above sea level.
        /// </summary>
        [TestMethod]
        public void TestRatesElevation()
        {
            double delta = 1.0E-12;
            Common.xlon = 90.0;
            Common.ilatsw = 1;
            Common.ilattm[0] = 40;
            Common.aveozo = -1.0;
            Common.ioz = 0;
            Common.q = 0.32;
            Common.deltaz = 0.001;
            Common.elevation = 4.0;
            Common.dinc = 10.0;
            Common.dfinal = 5.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Spring";
            Common.minwav = 1;
            Common.maxwav = 19;

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            int startPos;
            int length;

            string strr1 = dtday.Rows[19]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[20]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[21]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);

            double r1, r2, r3;
            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(1.72E-07, r1, delta);
            Assert.AreEqual(8.21E-03, r2, delta);
            Assert.AreEqual(8.45E+01, r3, delta);
        }

        /// <summary>
        /// Compute direct photolysis rates for carbazole as a 
        /// function of time-of-day for latitudes 30 degrees and 
        /// 40 degrees north, longitude 90 degrees west during 
        /// mid-summer and mid-winter.
        /// </summary>
        [TestMethod]
        public void TestRatesForLatsSeasons()
        {
            double delta = 1.0E-12;
            Common.xlon = 90.0;
            Common.ilatsw = 2;
            Common.ilattm[0] = 30.0;
            Common.ilattm[1] = 50.0;
            Common.aveozo = -1.0;
            Common.ioz = 0;
            Common.q = 0.0033;
            Common.deltaz = 0.001;
            Common.elevation = 0.0;
            Common.dinc = 10.0;
            Common.dfinal = 5.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Summer";
            Common.sease[1] = "Winter";
            Common.iseasw = 2;
            Common.minwav = 8;
            Common.maxwav = 25;
            Common.setEppest(5540, 7);
            Common.setEppest(3100, 8);
            Common.setEppest(2440, 9);
            Common.setEppest(2270, 10);
            Common.setEppest(2390, 11);
            Common.setEppest(2530, 12);
            Common.setEppest(2600, 13);
            Common.setEppest(2700, 14);
            Common.setEppest(2920, 15);
            Common.setEppest(3190, 16);
            Common.setEppest(3170, 17);
            Common.setEppest(2900, 18);
            Common.setEppest(1520, 19);
            Common.setEppest(166, 20);
            Common.setEppest(23, 21);
            Common.setEppest(13, 22);
            Common.setEppest(12, 23);
            Common.setEppest(2, 24);

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            int startPos;
            int length;

            string strr1 = dtday.Rows[23]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[24]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[25]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);

            double r1, r2, r3;
            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(6.36E-05, r1, delta);
            Assert.AreEqual(3.14E+00, r2, delta);
            Assert.AreEqual(0.22E+00, r3, delta);

            strr1 = dtday.Rows[39]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[40]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[41]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(3.50E-05, r1, delta);
            Assert.AreEqual(1.32E+00, r2, delta);
            Assert.AreEqual(0.524E+00, r3, delta);

            strr1 = dtday.Rows[61]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[62]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[63]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(5.06E-05, r1, delta);
            Assert.AreEqual(2.84E+00, r2, delta);
            Assert.AreEqual(0.244E+00, r3, delta);

            strr1 = dtday.Rows[73]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[74]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[75]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(1.11E-05, r1, delta);
            Assert.AreEqual(0.348E+00, r2, delta);
            Assert.AreEqual(1.99E+00, r3, delta);
            
        }

        /// <summary>
        /// Compute direct photolysis rate constants for carbazole for 
        /// the date June 21, 1976 at latitude 34.00 degrees north, 
        /// longitude 83.2 degrees west near the surface of a water body 
        /// (0.001 CM).
        /// </summary>
        [TestMethod]
        public void TestRateAtSpecificTimeLocation()
        {
            double delta = 1.0E-12;
            Common.xlon = 83.2;
            Common.ilatsw = 2;
            Common.ilattm[0] = 30.0;
            Common.ilattm[1] = 50.0;
            Common.aveozo = 0.3;
            Common.ioz = 1;
            Common.q = 0.0033;
            Common.deltaz = 0.001;
            Common.elevation = 0.0;
            Common.dinc = 10.0;
            Common.dfinal = 5.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Summer";
            Common.sease[1] = "Winter";
            Common.iseasw = 2;
            Common.minwav = 8;
            Common.maxwav = 25;
            Common.setEppest(5540, 7);
            Common.setEppest(3100, 8);
            Common.setEppest(2440, 9);
            Common.setEppest(2270, 10);
            Common.setEppest(2390, 11);
            Common.setEppest(2530, 12);
            Common.setEppest(2600, 13);
            Common.setEppest(2700, 14);
            Common.setEppest(2920, 15);
            Common.setEppest(3190, 16);
            Common.setEppest(3170, 17);
            Common.setEppest(2900, 18);
            Common.setEppest(1520, 19);
            Common.setEppest(166, 20);
            Common.setEppest(23, 21);
            Common.setEppest(13, 22);
            Common.setEppest(12, 23);
            Common.setEppest(2, 24);

            Common.xx[0] = 23;
            Common.xx[1] = 26;
            Common.xx[2] = 24.1;
            Common.xx[3] = 5;
            Common.xx[4] = 58;
            Common.xx[5] = 53.26;
            Common.xx[6] = 17;
            Common.xx[7] = 57;
            Common.xx[8] = 16.047;
            Common.typlat = 34.0;
            Common.ityp = 1;
            Common.ilatsw = 1;

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            int startPos;
            int length;

            string strr1 = dtday.Rows[21]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[22]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[23]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);

            double r1, r2, r3;
            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(6.26E-05, r1, delta);
            Assert.AreEqual(3.24E+00, r2, delta);
            Assert.AreEqual(0.214E+00, r3, delta);
        }

        /// <summary>
        /// Compute average direct photolysis rate constants for 
        /// carbazole in well mixed layers of various depths. 
        /// Use time and place specified in PART V.
        /// </summary>
        [TestMethod]
        public void TestRatesAtVariousDepthsInMixedLayers()
        {
            double delta = 1.0E-12;
            Common.xlon = 83.20;
            Common.ilatsw = 2;
            Common.ilattm[0] = 30.0;
            Common.ilattm[1] = 50.0;
            Common.aveozo = 0.3;
            Common.ioz = 1;
            Common.q = 0.0033;
            Common.deltaz = 0.0;
            Common.elevation = 0.0;
            Common.dinc = 1000.0;
            Common.dfinal = 4000.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Summer";
            Common.iseasw = 1;
            Common.minwav = 8;
            Common.maxwav = 25;
            Common.setEppest(5540, 7);
            Common.setEppest(3100, 8);
            Common.setEppest(2440, 9);
            Common.setEppest(2270, 10);
            Common.setEppest(2390, 11);
            Common.setEppest(2530, 12);
            Common.setEppest(2600, 13);
            Common.setEppest(2700, 14);
            Common.setEppest(2920, 15);
            Common.setEppest(3190, 16);
            Common.setEppest(3170, 17);
            Common.setEppest(2900, 18);
            Common.setEppest(1520, 19);
            Common.setEppest(166, 20);
            Common.setEppest(23, 21);
            Common.setEppest(13, 22);
            Common.setEppest(12, 23);
            Common.setEppest(2, 24);

            Common.xx[0] = 23;
            Common.xx[1] = 26;
            Common.xx[2] = 24.1;
            Common.xx[3] = 5;
            Common.xx[4] = 58;
            Common.xx[5] = 53.26;
            Common.xx[6] = 17;
            Common.xx[7] = 57;
            Common.xx[8] = 16.047;
            Common.typlat = 34.0;
            Common.ityp = 1;
            Common.ilatsw = 1;

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            // Depth = 0.001cm
            int startPos;
            int length;

            string strr1 = dtday.Rows[21]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[22]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[23]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);

            double r1, r2, r3;
            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(6.26E-05, r1, delta);
            Assert.AreEqual(3.24E+00, r2, delta);
            Assert.AreEqual(0.214E+00, r3, delta);

            // Depth = 1000.0cm
            strr1 = dtday.Rows[45]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[46]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[47]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(4.30E-05, r1, delta);
            Assert.AreEqual(2.23E+00, r2, delta);
            Assert.AreEqual(0.311E+00, r3, delta);

            // Depth = 2000.0cm
            strr1 = dtday.Rows[69]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[70]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[71]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(3.13E-05, r1, delta);
            Assert.AreEqual(1.62E+00, r2, delta);
            Assert.AreEqual(0.427E+00, r3, delta);

            // Depth = 3000.0cm
            strr1 = dtday.Rows[93]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[94]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[95]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(2.40E-05, r1, delta);
            Assert.AreEqual(1.24E+00, r2, delta);
            Assert.AreEqual(0.558E+00, r3, delta);

            // Depth = 4000.0cm
            strr1 = dtday.Rows[117]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[118]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[119]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(1.91E-05, r1, delta);
            Assert.AreEqual(0.989E+00, r2, delta);
            Assert.AreEqual(0.701E+00, r3, delta);
        }

        /// <summary>
        /// Compute direct photolysis rate constants for 
        /// carbazole at various depths in a water column. 
        /// Use time and place specified in PART V.
        /// </summary>
        [TestMethod]
        public void TestRatesAtVariousDepths()
        {
            double delta = 1.0E-12;
            Common.xlon = 83.20;
            Common.ilatsw = 2;
            Common.ilattm[0] = 30.0;
            Common.ilattm[1] = 50.0;
            Common.aveozo = 0.3;
            Common.ioz = 1;
            Common.q = 0.0033;
            Common.deltaz = 0.001;
            Common.useDeltaz = true;
            Common.elevation = 0.0;
            Common.dinc = 1000.0;
            Common.dfinal = 4000.0;
            Common.dinit = 0.001;
            Common.musubr = 1.34;
            Common.sease[0] = "Summer";
            Common.iseasw = 1;
            Common.minwav = 8;
            Common.maxwav = 25;
            Common.setEppest(5540, 7);
            Common.setEppest(3100, 8);
            Common.setEppest(2440, 9);
            Common.setEppest(2270, 10);
            Common.setEppest(2390, 11);
            Common.setEppest(2530, 12);
            Common.setEppest(2600, 13);
            Common.setEppest(2700, 14);
            Common.setEppest(2920, 15);
            Common.setEppest(3190, 16);
            Common.setEppest(3170, 17);
            Common.setEppest(2900, 18);
            Common.setEppest(1520, 19);
            Common.setEppest(166, 20);
            Common.setEppest(23, 21);
            Common.setEppest(13, 22);
            Common.setEppest(12, 23);
            Common.setEppest(2, 24);

            Common.xx[0] = 23;
            Common.xx[1] = 26;
            Common.xx[2] = 24.1;
            Common.xx[3] = 5;
            Common.xx[4] = 58;
            Common.xx[5] = 53.26;
            Common.xx[6] = 17;
            Common.xx[7] = 57;
            Common.xx[8] = 16.047;
            Common.typlat = 34.0;
            Common.ityp = 1;
            Common.ilatsw = 1;

            DataTable dtday = new DataTable();
            DataTable dtKL = new DataTable();
            Depend dp = new Depend();
            string errorMsg = "";
            dp.CalculatePhotolysisRatesHalfLivesTDay(out dtday, out dtKL, out errorMsg);

            // Depth = 0.001cm
            int startPos;
            int length;

            string strr1 = dtday.Rows[21]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            string sub1 = strr1.Substring(startPos, length);

            string strr2 = dtday.Rows[22]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            string sub2 = strr2.Substring(startPos, length);

            string strr3 = dtday.Rows[23]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            string sub3 = strr3.Substring(startPos, length);

            double r1, r2, r3;
            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(6.26E-05, r1, delta);
            Assert.AreEqual(3.24E+00, r2, delta);
            Assert.AreEqual(0.214E+00, r3, delta);

            // Depth = 1000.0cm
            strr1 = dtday.Rows[45]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[46]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[47]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(2.82E-05, r1, delta);
            Assert.AreEqual(1.46E+00, r2, delta);
            Assert.AreEqual(0.475E+00, r3, delta);

            // Depth = 2000.0cm
            strr1 = dtday.Rows[69]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[70]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[71]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(1.31E-05, r1, delta);
            Assert.AreEqual(0.680E+00, r2, delta);
            Assert.AreEqual(1.02E+00, r3, delta);

            // Depth = 3000.0cm
            strr1 = dtday.Rows[93]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[94]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[95]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(6.30E-06, r1, delta);
            Assert.AreEqual(0.326E+00, r2, delta);
            Assert.AreEqual(2.130E+00, r3, delta);

            // Depth = 4000.0cm
            strr1 = dtday.Rows[117]["Summary Items"].ToString();
            startPos = strr1.IndexOf("=") + 2;
            length = strr1.Length - startPos;
            sub1 = strr1.Substring(startPos, length);

            strr2 = dtday.Rows[118]["Summary Items"].ToString();
            startPos = strr2.IndexOf("=") + 2;
            length = strr2.Length - startPos;
            sub2 = strr2.Substring(startPos, length);

            strr3 = dtday.Rows[119]["Summary Items"].ToString();
            startPos = strr3.IndexOf("=") + 2;
            length = strr3.Length - startPos;
            sub3 = strr3.Substring(startPos, length);

            Double.TryParse(sub1, out r1);
            Double.TryParse(sub2, out r2);
            Double.TryParse(sub3, out r3);

            Assert.AreEqual(3.11E-06, r1, delta);
            Assert.AreEqual(0.161E+00, r2, delta);
            Assert.AreEqual(4.31E+00, r3, delta);
        }
    }
}
